package com.persagy.cache;

import java.util.LinkedList;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.constant.RedisConstant;
import com.persagy.entity.AlarmConditionState;
import com.persagy.entity.ZktAlarmRecordDO;
import com.persagy.entity.v2.AlarmCondition;
import com.persagy.entity.v2.ItemCodeCondition;
import com.persagy.entity.v2.ObjConditionInfo;
import com.persagy.repository.AlarmRecordRepository;

import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;

@Component
@Slf4j
public class AlarmRedisCache implements RedisCache {

	@Resource(name = "alarmRedisTemplate")
	private RedisTemplate<String, String> alarmRedisTemplate;

	@Autowired
	AlarmRecordRepository alarmRecordRepository;

	public AlarmConditionState getAlarmDefineState(String defineId) {
		AlarmConditionState alarmConditionState = null;
		try {
			Object obj = alarmRedisTemplate.opsForHash().get(RedisConstant.DEFINE_ID, defineId);
			if (null != obj && StringUtils.isNotBlank(obj.toString())) {
				try {
					alarmConditionState =JSONUtil.toBean(obj.toString(), AlarmConditionState.class);
				} catch (Exception e) {
					log.error(" alarmDefineState json 转换 AlarmConditionState 失败", e);
				}
			} else {
				// 默认正常报警状态
				Optional<ZktAlarmRecordDO> recordOptional = alarmRecordRepository.findById(defineId);
				if (recordOptional.isPresent()) {
					ZktAlarmRecordDO alarmRecordDO = recordOptional.get();
					// 数据库报警状态：1-未处理
					if ("1".equals(alarmRecordDO.getState())) {
						alarmConditionState = new AlarmConditionState(defineId);
						alarmConditionState.setState(AlarmConditionState.State.NOT_DEAL.getType());
						alarmConditionState.setAlarmStartTime(alarmRecordDO.getAlarmTime());
					}
				} else {
					alarmConditionState = new AlarmConditionState(defineId);
				}
			}
			log.debug("alarmDefineState: [{}], （ 0-正常 1-报警）", alarmConditionState.getState());
		} catch (Exception e) {
			log.warn("从redis获取报警定义状态失败" + e);
		}
		return alarmConditionState;
	}

	public void setAlarmState(String defineId, AlarmConditionState alarmConditionState) {
		alarmRedisTemplate.opsForHash().put(RedisConstant.DEFINE_ID, defineId, JSON.toJSONString(alarmConditionState));
	}

	public AlarmCondition getAlarmConditionRedis(String conditionId) {
		String str = alarmRedisTemplate.execute((RedisCallback<String>) conn -> {
			conn.select(14);
			byte[] condition = conn.hGet(RedisConstant.ALARMCONDITION_ID.getBytes(), conditionId.getBytes());
			return new String(condition);
		});
		JSONObject object = JSONObject.parseObject(str);
		AlarmCondition alarmCondition = JSON.toJavaObject(object, AlarmCondition.class);
		return alarmCondition;
	}

	public ObjConditionInfo getAlarmConditionsByObjId(String meterId) {
		try {
			String str = alarmRedisTemplate.opsForHash().get(RedisConstant.METER_ID, meterId).toString();
			if (StringUtils.isNotBlank(str)) {
				JSONObject object = JSONObject.parseObject(str);
				JSONArray conditions = object.getJSONArray("conditions");
				if (null == conditions || conditions.isEmpty()) {
					return null;
				}
				ObjConditionInfo objConditionInfo = JSON.toJavaObject(object, ObjConditionInfo.class);
				List<ItemCodeCondition> iccArrayy = JSON.parseArray(conditions.toString(), ItemCodeCondition.class);
				objConditionInfo.setConditions(new LinkedList<>(iccArrayy));
				return objConditionInfo;
			}
		} catch (Exception e) {
			log.warn("从redis获取设备的报警条件信息失败" + e);
		}
		return null;
	}

	public AlarmCondition getAlarmCondition(String conditionId) {
		try {
			Object obj = alarmRedisTemplate.opsForHash().get(RedisConstant.ALARMCONDITION_ID, conditionId);
			if (null != obj && StringUtils.isNotBlank(obj.toString())) {
				JSONObject object = JSONObject.parseObject(obj.toString());
				AlarmCondition alarmCondition = JSON.toJavaObject(object, AlarmCondition.class);
				return alarmCondition;
			}
		} catch (Exception e) {
			log.warn("从redis获取报警条件失败" + e);
		}
		return null;
	}

	/**
	 * 释放锁
	 * 
	 * @param key
	 */
	public void releaseLock(String key) {
		alarmRedisTemplate.delete(key);
	}

	/**
	 * 加锁
	 * 
	 * @param key
	 * @param value
	 * @param timeout 过期时间
	 */
	public boolean lock(String key, String value, Integer timeout) {
		boolean bool;
		int i = 1;
		while (true) {
			Boolean b = alarmRedisTemplate.opsForValue().setIfAbsent(key, value, timeout, TimeUnit.SECONDS);
			System.out.println(key + "---key----" + b);
			if (b) {
				bool = b;
				break;
			} else {
				try {
					Thread.sleep(100);
				} catch (InterruptedException e) {
					Thread.currentThread().interrupt();
				}
			}
			System.out.println(i);
		}
		return bool;

	}

}
