package com.persagy.entity;


import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.enumeration.YesNoEnum;
import com.vladmihalcea.hibernate.type.json.JsonStringType;
import io.swagger.annotations.ApiModel;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.annotations.Type;
import org.hibernate.annotations.TypeDef;

import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import java.io.Serializable;

/**
 * 缓存报警持续时间
 */
@ApiModel(value = "缓存报警持续时间")
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@Entity
@Table(name = "alarm_last_time")
@TypeDef(name = "json", typeClass = JsonStringType.class)
public class AlarmLastTime implements Serializable {
    private static final long serialVersionUID = 1L;
    /**
     * 报警记录ID
     */
    @Id
    private String id;
    /**
     * 持续时间
     */
    private Integer lastTime;
    /**
     * 报警持续时间消息是否已发送
     */
    private YesNoEnum hasSent;
    /**
     * 报警记录是否已创建
     */
    private YesNoEnum alarmHasCreated;
    /**
     * 报警记录是否可删除
     */
    private YesNoEnum deleteAvailable;
    /**
     * 项目id
     */
    private String projectId;
    /**
     * 要发送的报警持续消息
     */
    @Type(type = "json")
    private ObjectNode alarmLastMsg;

    @Override
    public String toString() {
        return "lastTime: " + this.getLastTime() + "; " +
                "hasSent: " + this.getHasSent() + "; " +
                "alarmHasCreated: " + this.getAlarmHasCreated() + "; " +
                "deleteAvailable: " + this.getDeleteAvailable() + ";";
    }
}